<?php
/**
 * The icon settings page.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage;

use Locr\WordPress\Plugin\LocrMaps\Admin\{Database, IconListTable, SettingsPage};
use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\Icon as EntitiesIcon;
use Locr\WordPress\Plugin\LocrMaps\DDD\Repositories\Icons as IconsRepository;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon\{Id, Size, Url};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\IconOptions;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\IconOptions\{IconName, SetIconOnInitialPosition};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\SettingsMessageId;
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for rendering the icon settings page.
 */
final class Icons {
	private const OPTION_GROUP = Plugin::SLUG . '_icons_options_group';
	private const SECTION      = Plugin::SLUG . '_icons_section';
	public const SUB_PAGE_NAME = Plugin::SLUG . '-icons';

	/**
	 * This function initializes the icon settings page.
	 */
	public static function init(): void {
		self::handle_delete_icon_item_action();
		add_action( 'admin_enqueue_scripts', array( self::class, 'enqueue_scripts' ) );
	}

	/**
	 * This function adds the main menu.
	 */
	public static function add_submenu(): void {
		add_submenu_page(
			parent_slug: SettingsPage::MAIN_PAGE,
			page_title: __( 'Icons', 'locrmaps-textdomain' ),
			menu_title: __( 'Icons', 'locrmaps-textdomain' ),
			capability: 'manage_options',
			menu_slug: self::SUB_PAGE_NAME,
			callback: array( self::class, 'render_settings_page' )
		);

		do_action( 'locrmaps_after_added_icon_submenu' );
	}

	/**
	 * This function enqueues the scripts.
	 *
	 * @param string $hook The hook.
	 */
	public static function enqueue_scripts( $hook ): void {
		if ( 'locrmaps_page_' . self::SUB_PAGE_NAME !== $hook ) {
			return;
		}

		wp_enqueue_media();
		wp_enqueue_script(
			handle: Plugin::SLUG . '-media-uploader',
			src: plugin_dir_url( dirname( dirname( __DIR__ ) ) ) . 'public/js/media_uploader.js',
			deps: array( 'jquery' ),
			ver: md5_file( plugin_dir_path( dirname( dirname( __DIR__ ) ) ) . 'public/js/media_uploader.js' ),
			args: array( 'in_footer' => true )
		);
	}

	/**
	 * This function handles the delete icon item action.
	 */
	public static function handle_delete_icon_item_action(): void {
		$page   = (string) ( filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? '' );
		$action = (string) ( filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? '' );

		if ( self::SUB_PAGE_NAME !== $page || 'delete_icon_item' !== $action ) {
			return;
		}

		$admin_url = admin_url( 'admin.php?page=' . self::SUB_PAGE_NAME );

		$wp_nonce = (string) ( filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? '' );
		if ( ! wp_verify_nonce( $wp_nonce, 'delete_icon_item' ) ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
			exit;
		}

		$icon_id = intval( filter_input( INPUT_GET, 'icon_id', FILTER_VALIDATE_INT ) ?? 0 );
		if ( ! $icon_id ) {
			return;
		}

		global $wpdb;

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		$table_name = $table_prefix . 'locrmaps_icons';
		// @phpstan-ignore method.nonObject
		$deleted = $wpdb->delete(
			table: $table_name,
			where: array( 'id' => $icon_id ),
			where_format: array( '%d' )
		);
		if ( $deleted ) {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::IconDeleted->value, $admin_url ) );
			exit;
		} else {
			wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::IconNotDeleted->value, $admin_url ) );
			exit;
		}
	}

	/**
	 * This function handles the upload icon.
	 */
	public static function handle_upload_icon(): void {
		$icon_url = (string) ( filter_input( INPUT_POST, 'icon_url', FILTER_DEFAULT ) ?? '' );
		if ( empty( $icon_url ) ) {
			return;
		}

		$icon_url  = sanitize_text_field( wp_unslash( $icon_url ) );
		$icon_data = wp_remote_get( $icon_url, array( 'sslverify' => false ) );
		if ( ! is_array( $icon_data ) ) {
			add_settings_error(
				setting: IconOptions::OPTION_NAME,
				code: SettingsPage::ERROR_SLUG_NAME_ICONS,
				message: __( 'The icon could not be added (url-error).', 'locrmaps-textdomain' ),
				type: 'error'
			);
			return;
		}

		$image = imagecreatefromstring( $icon_data['body'] );
		if ( false === $image ) {
			add_settings_error(
				setting: IconOptions::OPTION_NAME,
				code: SettingsPage::ERROR_SLUG_NAME_ICONS,
				message: __( 'The icon could not be added (image-error).', 'locrmaps-textdomain' ),
				type: 'error'
			);
			return;
		}

		$icon = new EntitiesIcon(
			id: new Id( PHP_INT_MAX ),
			url: new Url( $icon_url ),
			size: new Size( width: imagesx( $image ), height: imagesy( $image ) ),
		);

		$icons_repository = new IconsRepository();
		try {
			$icons_repository->add( $icon );

			add_settings_error(
				setting: IconOptions::OPTION_NAME,
				code: SettingsPage::ERROR_SLUG_NAME_ICONS,
				message: __( 'The icon was added.', 'locrmaps-textdomain' ),
				type: 'success'
			);
		} catch ( \Exception $exc ) {
			add_settings_error(
				setting: IconOptions::OPTION_NAME,
				code: SettingsPage::ERROR_SLUG_NAME_ICONS,
				message: __( 'The icon could not be added (db-error).', 'locrmaps-textdomain' ),
				type: 'error'
			);
		}
	}

	/**
	 * This function renders the icon name field.
	 */
	public static function icon_name_callback(): void {
		$icons_repository = new IconsRepository();
		$icons            = $icons_repository->get_all_icons();
		$options          = new IconOptions( only_option: IconName::OPTION_KEY );

		print '<select class="regular-text" name="' . esc_attr( IconOptions::OPTION_NAME . '[' . IconName::OPTION_KEY . ']' ) . '" id="' . esc_attr( IconName::OPTION_KEY ) . '">';
		print '<option value="">-- ' . esc_html__( 'Select an icon', 'locrmaps-textdomain' ) . ' --</option>';
		foreach ( $icons as $icon ) {
			if ( $icon instanceof EntitiesIcon ) {
				print '<option value="' . esc_attr( (string) $icon->name ) . '"';
				if ( (string) $icon->name === (string) $options->icon_name ) {
					print ' selected="selected"';
				}
				print '>' . esc_html( (string) $icon->name ) . '</option>';
			}
		}
		print '</select>';
	}

	/**
	 * This function renders the set icon on initial position field.
	 */
	public static function set_icon_on_initial_position_callback(): void {
		$options = new IconOptions( only_option: SetIconOnInitialPosition::OPTION_KEY );

		$checkbox = '<input type="checkbox" name="' . esc_attr( IconOptions::OPTION_NAME . '[' . SetIconOnInitialPosition::OPTION_KEY . ']' ) . '" id="' . esc_attr( SetIconOnInitialPosition::OPTION_KEY ) . '"';
		if ( $options->set_icon_on_initial_position->get_value() ) {
			$checkbox .= ' checked="checked"';
		}
		$checkbox .= '>';

		print $checkbox; // @phpcs:ignore
	}

	/**
	 * This function registers the settings.
	 */
	public static function register_settings(): void {
		register_setting( self::OPTION_GROUP, IconOptions::OPTION_NAME );

		add_settings_section(
			id: self::SECTION,
			title: __( 'Icon settings', 'locrmaps-textdomain' ),
			callback: function () {},
			page: self::SUB_PAGE_NAME
		);

		add_settings_field(
			id: SetIconOnInitialPosition::OPTION_KEY,
			title: __( 'Set icon on initial position', 'locrmaps-textdomain' ) . ':',
			callback: array( self::class, 'set_icon_on_initial_position_callback' ),
			page: self::SUB_PAGE_NAME,
			section: self::SECTION,
			args: array( 'label_for' => SetIconOnInitialPosition::OPTION_KEY )
		);

		add_settings_field(
			id: IconName::OPTION_KEY,
			title: __( 'Icon name', 'locrmaps-textdomain' ) . ':',
			callback: array( self::class, 'icon_name_callback' ),
			page: self::SUB_PAGE_NAME,
			section: self::SECTION,
			args: array( 'label_for' => IconName::OPTION_KEY )
		);
	}

	/**
	 * This function renders the settings page for the icons.
	 */
	public static function render_settings_page(): void {
		Database::init_icons_table();

		$table = new IconListTable();

		self::handle_upload_icon();

		print '<div class="wrap">' . PHP_EOL;
		print '<h1>' . esc_html__( 'locrMAPS', 'locrmaps-textdomain' ) . ' &rsaquo; ' . esc_html__( 'Icons', 'default' ) . '</h1>' . PHP_EOL;

		settings_errors();

		print '</div>' . PHP_EOL;

		/**
		 * Upload or select icons.
		 */
		print '<hr/>' . PHP_EOL;
		print '<div class="wrap">' . PHP_EOL;
		print '<form id="' . esc_attr( self::SUB_PAGE_NAME . '-form' ) . '" method="post">' . PHP_EOL;
		print '<div>' . PHP_EOL;
		if ( 'edit_icon_item' !== $table->current_action() ) {
			print '<button id="upload_icon_button" class="button">' . esc_html__( 'Upload/Add image', 'locrmaps-textdomain' ) . '</button>' . PHP_EOL;
		}
		print '<input type="hidden" id="icon_url" name="icon_url" value=""/>' . PHP_EOL;
		print '</div>' . PHP_EOL;

		if ( 'edit_icon_item' === $table->current_action() ) {
			$page   = (string) ( filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? '' );
			$action = (string) ( filter_input( INPUT_GET, 'action', FILTER_DEFAULT ) ?? '' );

			if ( self::SUB_PAGE_NAME !== $page || 'edit_icon_item' !== $action ) {
				return;
			}

			$admin_url = add_query_arg( 'page', self::SUB_PAGE_NAME, admin_url( 'admin.php' ) );

			$wp_nonce = (string) ( filter_input( INPUT_GET, '_wpnonce', FILTER_DEFAULT ) ?? '' );
			if ( ! wp_verify_nonce( $wp_nonce, 'edit_icon_item' ) ) {
				wp_safe_redirect( add_query_arg( 'message', SettingsMessageId::InvalidNonce->value, $admin_url ) );
				exit;
			}

			$icon_id = intval( filter_input( INPUT_GET, 'icon_id', FILTER_VALIDATE_INT ) ?? 0 );
			if ( $icon_id ) {
				$table->display_edit_form( $icon_id );
			} else {
				print '<div class="error"><p>' . esc_html__( 'No "icon_id"!', 'locrmaps-textdomain' ) . '</p></div>' . PHP_EOL;
			}
		} else {
			$table->prepare_items();
			$table->display();
		}
		print '</form>' . PHP_EOL;
		print '</div>' . PHP_EOL;
	}
}
