<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Repositories;

use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Id;
use Locr\WordPress\Plugin\LocrMaps\{Plugin, TypedSequence};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icons.
 */
final class OnlineMaps {
	/**
	 * This function adds an ONLINEmap into the database.
	 *
	 * @param OnlineMap $onlinemap The ONLINEmap to add.
	 * @throws \Exception If the ONLINEmap could not be added.
	 */
	public function add( OnlineMap $onlinemap ): void {
		global $wpdb;

		$insert_data   = array(
			'name' => (string) $onlinemap->name,
		);
		$insert_format = array( '%s' );

		// @phpstan-ignore method.nonObject
		$insert_result = $wpdb->insert(
			table: $this->get_tablename(),
			data: $insert_data,
			format: $insert_format
		);

		if ( false === $insert_result ) {
			throw new \Exception( 'The ONLINEmap could not be added (db-error).' );
		}

		// @phpstan-ignore property.nonObject
		$onlinemap_id = new Id( $wpdb->insert_id );
		$onlinemap->update_id( $onlinemap_id );
	}

	/**
	 * This function deletes an ONLINEmap by id.
	 *
	 * @param Id $id The id of the ONLINEmap.
	 * @throws \Exception If the ONLINEmap could not be deleted.
	 */
	public function delete_by_id( Id $id ): void {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$deleted = $wpdb->delete(
			table: $this->get_tablename(),
			where: array( 'id' => $id->value ),
			where_format: array( '%d' )
		);
		if ( false === $deleted ) {
			throw new \Exception( 'The ONLINEmap could not be deleted (db-error).' );
		}
	}

	/**
	 * This function deletes all icons by ids.
	 *
	 * @param array<mixed> $ids The ids of the icons.
	 * @throws \Exception If the icons could not be deleted.
	 */
	public function delete_all_ids( array $ids ): void {
		global $wpdb;

		foreach ( $ids as $id ) {
			// @phpstan-ignore method.nonObject
			$deleted = $wpdb->delete(
				table: $this->get_tablename(),
				where: array( 'id' => $id ),
				where_format: array( '%d' )
			);
			if ( false === $deleted ) {
				throw new \Exception( 'The ONLINEmaps could not be deleted at all (db-error).' );
			}
		}
	}

	/**
	 * This function gets all icons.
	 *
	 * @return TypedSequence The icons.
	 */
	public function get_all(): TypedSequence {
		global $wpdb;

		$onlinemaps = TypedSequence::for_type( OnlineMap::class );

		$tablename = $this->get_tablename();
		// @phpstan-ignore method.nonObject
		$db_result = $wpdb->get_results( "SELECT * FROM {$tablename}", ARRAY_A ); // phpcs:ignore
		if ( is_array( $db_result ) ) {
			foreach ( $db_result as $row ) {
				$onlinemaps->add( OnlineMap::from_db_row( $row ) );
			}
		}

		return $onlinemaps;
	}

	/**
	 * This function gets an ONLINEmap by id.
	 *
	 * @param Id $id The id of the ONLINEmap.
	 */
	public function get_by_id( Id $id ): ?OnlineMap {
		global $wpdb;

		$tablename = $this->get_tablename();
		// @phpstan-ignore method.nonObject
		$db_result = $wpdb->get_row(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare(
				"SELECT * FROM {$tablename} WHERE id = %s", // phpcs:ignore
				$id->value
			),
			ARRAY_A
		);

		if ( is_null( $db_result ) ) {
			return null;
		}

		return OnlineMap::from_db_row( $db_result );
	}

	/**
	 * This function gets the table name.
	 */
	public function get_tablename(): string {
		global $wpdb;

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		return $table_prefix . Plugin::SLUG . '_onlinemaps';
	}

	/**
	 * This function updates an ONLINEmap.
	 *
	 * @param OnlineMap $onlinemap The ONLINEmap to update.
	 * @throws \Exception If the ONLINEmap could not be updated.
	 */
	public function update( OnlineMap $onlinemap ): void {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$updated = $wpdb->update(
			table: $this->get_tablename(),
			data: array(
				'name'   => sanitize_text_field( (string) $onlinemap->name ),
				'style'  => sanitize_text_field( (string) $onlinemap->style ),
				'width'  => sanitize_text_field( (string) $onlinemap->width ),
				'height' => sanitize_text_field( (string) $onlinemap->height ),
				'border' => $onlinemap->build_border_json(),
			),
			where: array( 'id' => $onlinemap->id->value ),
		);

		if ( false === $updated ) {
			throw new \Exception( 'The ONLINEmap could not be updated (db-error).' );
		}
	}
}
